<?php

if ( ! function_exists('data_get')) {
    /**
     * @param $target
     * @param $key
     * @param $default
     * @return mixed
     */
    function data_get($target, $key, $default = null)
    {
        if (is_null($key)) {
            return $target;
        }

        foreach (explode('.', $key) as $segment) {
            if (is_array($target)) {
                if ( ! array_key_exists($segment, $target)) {
                    return $default;
                }

                $target = $target[$segment];
            } elseif (is_object($target)) {
                if ( ! isset($target->{$segment})) {
                    return $default;
                }

                $target = $target->{$segment};
            } else {
                return $default;
            }
        }

        return $target;
    }
}

if ( ! function_exists('str_contains')) {
    /**
     * @param $string
     * @param $search
     */
    function str_contains($string, $search)
    {
        foreach ((array) $search as $needle) {
            if ($needle !== '' && mb_strpos($string, $needle) !== false) {
                return true;
            }
        }
        return false;
    }
}

if ( ! function_exists('str_after')) {
    /**
     * @param $string
     * @param $search
     * @return mixed
     */
    function str_after($string, $search)
    {
        return $search === '' ? $string : array_reverse(explode($search, $string, 2))[0];
    }
}

if ( ! function_exists('str_before')) {
    /**
     * @param $string
     * @param $search
     * @return mixed
     */
    function str_before($string, $search)
    {
        return $search === '' ? $string : explode($search, $string)[0];
    }
}

if ( ! function_exists('str_finish')) {
    /**
     * @param $value
     * @param $cap
     */
    function str_finish($value, $cap)
    {
        $quoted = preg_quote($cap, '/');
        return preg_replace('/(?:' . $quoted . ')+$/u', '', $value) . $cap;
    }
}

if ( ! function_exists('str_starts_with')) {
    /**
     * @param $haystack
     * @param $needles
     */
    function str_starts_with($haystack, $needles)
    {
        foreach ((array) $needles as $needle) {
            if ($needle !== '' && substr($haystack, 0, strlen($needle)) === (string) $needle) {
                return true;
            }
        }

        return false;
    }
}

if ( ! function_exists('number_raw')) {
    /**
     * @param $value
     */
    function number_raw($value)
    {
        return preg_replace('/[^0-9]/', '', $value);
    }
}

if ( ! function_exists('cache_remember_file')) {
    function cache_remember_file($cache_key, $value, $expiration = 300)
    {
        $path = dirname(__FILE__) . '/cache/';
        $file = $path . $cache_key;

        $current_time = time();
        if (file_exists($file) && (($current_time - $expiration) < filemtime($file)) && ! empty(file_get_contents($file))) {
            return json_decode(file_get_contents($file), true);
        } else {
            $dirname = dirname($file);
            if ( ! is_dir($dirname)) {
                mkdir($dirname, 0755, true);
            }
            $myfile = fopen($file, "w");
            fwrite($myfile, json_encode($value));
            fclose($myfile);
            return $value;
        }
    }
}

if ( ! function_exists('plyr_video')) {
    /**
     * @param $video
     * @param $type
     * @param null $poster
     * @return mixed
     */
    function plyr_video($video, $type = null, $poster = null)
    {
        $id = rand();
        wp_enqueue_style('plyr', 'https://cdn.plyr.io/3.6.1/plyr.css', []);
        wp_enqueue_script('plyr', 'https://cdn.plyr.io/3.6.1/plyr.js', []);

        switch ($type) {
            case 'youtube':
                $body = '<div id="' . $id . '" data-plyr-provider="youtube" data-plyr-embed-id="' . $video . '" poster="' . $poster . '"></div>';
                break;

            case 'vimeo':
                $body = '<div id="' . $id . '" data-plyr-provider="vimeo" data-plyr-embed-id="' . $video . '" poster="' . $poster . '"></div>';
                break;

            default:
                $body = '<video id="' . $id . '" playsinline controls data-poster="' . $poster . '">';
                $body .= '    <source src="' . $video . '" type="video/mp4" />';
                $body .= '</video>';
                break;
        }
        $body .= '<script>jQuery(document).ready(function($) { const player_' . $id . ' = new Plyr(document.getElementById("' . $id . '")); });</script>';
        return $body;
    }
}

if ( ! function_exists('get_time')) {
    /**
     * @param $value
     * @param $type
     * @return mixed
     */
    function get_time($value, $type)
    {
        switch ($type) {
            case 'minutes':
                return $value * 60;
                break;
            case 'hours':
                return $value * 60 * 60;
                break;
            case 'days':
                return $value * 60 * 60 * 24;
                break;
            case 'weeks':
                return $value * 60 * 60 * 24 * 7;
                break;
            case 'months':
                return $value * 60 * 60 * 24 * 30;
                break;

            default:
                return $value;
                break;
        }
    }
}

if ( ! function_exists('get_IP')) {
    /**
     * @param $getHostByAddr
     * @return mixed
     */
    function get_IP($getHostByAddr = false)
    {
        foreach (array('HTTP_CLIENT_IP', 'HTTP_X_FORWARDED_FOR', 'HTTP_X_FORWARDED', 'HTTP_X_CLUSTER_CLIENT_IP', 'HTTP_FORWARDED_FOR', 'HTTP_FORWARDED', 'REMOTE_ADDR') as $key) {
            if (array_key_exists($key, $_SERVER) === true) {
                foreach (array_map('trim', explode(',', $_SERVER[$key])) as $ip) {
                    if (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE) !== false) {
                        if ($getHostByAddr === true) {
                            return getHostByAddr($ip);
                        } else {
                            return $ip;
                        }
                    }
                }
            }
        }
        return json_decode(file_get_contents('https://get.geojs.io/v1/ip.json'), true)['ip'];
    }
}

if ( ! function_exists('wp_response')) {
    function wp_response($code, $value)
    {
        return new WP_REST_Response([
            'status'   => $code,
            'response' => $value,
        ]);
    }
}

if ( ! function_exists('cache_remember')) {
    function cache_remember($key, $content = null, $time = 60)
    {
        $cache = get_transient($key);
        if ($cache === false) {
            set_transient($key, $content, $time);
            return $content;
        }
        return $cache;
    }
}

if ( ! function_exists('player_video')) {
    function player_video($video, $type = null)
    {
        wp_register_style('am_player_video', false);
        wp_enqueue_style('am_player_video');
        wp_add_inline_style('am_player_video', '.embed-container { position: relative !important; padding-bottom: 56.25% !important; height: 0 !important; overflow: hidden !important; max-width: 100% !important; } .embed-container iframe, .embed-container object, .embed-container embed { position: absolute !important; top: 0 !important; left: 0 !important; width: 100% !important; height: 100% !important; }');

        switch ($type) {
            case 'youtube':
                $video = 'https://www.youtube.com/embed/' . $video;
                break;
            case 'vimeo':
                $video = 'https://player.vimeo.com/' . $video;
                break;

            default:
                break;
        }

        return "<div class='embed-container'>
            <iframe src='" . $video . "' frameborder='0' allow='autoplay; fullscreen; picture-in-picture' allowfullscreen></iframe>
        </div>";
    }
}

if ( ! function_exists('has_ssl')) {
    function has_ssl($url)
    {
        $stream = stream_context_create([
            'ssl' => [
                'capture_peer_cert' => true,
            ],
        ]);
        $read = fopen($url, 'rb', false, $stream);
        $cont = stream_context_get_params($read);
        return ( ! is_null($cont["options"]["ssl"]["peer_certificate"])) ? true : false;
    }
}
